/* Print the Simple and Resource values as shell variables
 *
 * Author: Bert Bos
 * Created: 18 August 2018
 *
 * Copyright © 2018 World Wide Web Consortium, (Massachusetts
 * Institute of Technology, European Research Consortium for
 * Informatics and Mathematics, Keio University, Beihang). All Rights
 * Reserved. See http://www.w3.org/Consortium/Legal/copyright-software
 */

#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#include "export.h"
#include "heap.e"
#include "types.e"
#include "propstore.e"
#include "xmlnames.e"


/* escape -- print a string with shell characters escaped */
static void escape(string s)
{
  for (;*s; s++)
    switch (*s) {
    case '"': printf("\\\""); break;
    case '\n': printf("\"$'\\n'\""); break;
    case '\r': printf("\"$'\\r'\""); break;
    default: putchar(*s);
    }
}

/* print_cb -- callback to print a single property */
static void print_cb(string property, PropertyType type, string value, Store sub,
		     string lang, void *data)
{
  int *n = (int*)data;
  string namespace, name, p;

  if (type == Simple || type == Resource) {

    /* Split the property name in a namespace and a local name. */
    for (name = p = property; *p;)
      if (!is_name_char(p, &p)) name = p;
    namespace = newnstring(property, name - property);

    /* Print Bash array variables */
    printf("xmp_name[%d]=\"", *n); escape(property); printf("\"\n");
    printf("xmp_namespace[%d]=\"", *n); escape(namespace); printf("\"\n");
    printf("xmp_localname[%d]=\"", *n); escape(name); printf("\"\n");
    printf("xmp_value[%d]=\"", *n); escape(value); printf("\"\n");
    printf("xmp_language[%d]=\"", *n); escape(lang); printf("\"\n");
    printf("xmp_type[%d]=%s\n", *n, type == Simple ? "Simple" : "Resource");

    dispose(namespace);
  }
}

    
/* print_sh -- print Simple and Resource values as Bash variables */
EXPORT void print_sh(Store s)
{
  int n = 0;

  store_walk(s, print_cb, &n);
}
