/* is_name_char() and is_name_start_char check XML syntax.
 *
 * is_name_char(string s, string *t) checks the first character of s
 * to see if it is a valid XML name character and returns true or
 * false. If t is not NULL, it also sets t to point to the next
 * character in s. The strings are asumed to be encoded in UTF-8.
 *
 * is_name_start_char() is similar, but checks for an XML name start
 * character.
 *
 * TODO: Check UTF-8 properly, instead of accepting all non-ASCII.
 *
 * Copyright © 2018 World Wide Web Consortium
 * See http://www.w3.org/Consortium/Legal/copyright-software
 *
 * Author: Bert Bos
 * Created: 16 August 2018
 */

#include "config.h"
#include <stdbool.h>
#include "export.h"
#include "types.e"


/* is_name_char -- true if s starts with a name character, return the rest */
EXPORT bool is_name_char(string s, string *tail)
{
  if (tail) *tail = s + 1;
  return ('A' <= *s && *s <= 'Z') || *s == '_' || ('a' <= *s && *s <= 'z') ||
    0200 <= (unsigned char) *s || *s == '-' || *s == '.' ||
    ('0' <= *s && *s <= '9');
}


/* is_name_start_char -- true if s starts with a name start character */
EXPORT bool is_name_start_char(string s, string *tail)
{
  if (tail) *tail = s + 1;
  return ('A' <= *s && *s <= 'Z') || *s == '_' ||
    ('a' <= *s && *s <= 'z') || 0200 <= (unsigned char)*s;
}
